@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -42,7 +42,7 @@ void
 String::setBuffer(char *aBuf, String::size_type aSize)
 {
     assert(undefined());
-    assert(aSize <= SizeMax_);
+    assert(aSize < 65536);
     buf_ = aBuf;
     size_ = aSize;
 }
@@ -105,7 +105,7 @@ void
 String::allocAndFill(const char *str, int len)
 {
     PROF_start(StringAllocAndFill);
-    assert(str);
+    assert(this && str);
     allocBuffer(len + 1);
     len_ = len;
     memcpy(buf_, str, len);
@@ -127,6 +127,7 @@ void
 String::clean()
 {
     PROF_start(StringClean);
+    assert(this);
 
     /* TODO if mempools has already closed this will FAIL!! */
     if (defined())
@@ -162,6 +163,7 @@ String::reset(char const *str)
 void
 String::append( char const *str, int len)
 {
+    assert(this);
     assert(str && len >= 0);
 
     PROF_start(StringAppend);
@@ -171,7 +173,7 @@ String::append( char const *str, int len)
     } else {
         // Create a temporary string and absorb it later.
         String snew;
-        assert(canGrowBy(len)); // otherwise snew.len_ may overflow below
+        assert(len_ + len < 65536); // otherwise snew.len_ overflows below
         snew.len_ = len_ + len;
         snew.allocBuffer(snew.len_ + 1);
 
