@@ -1,13 +1,20 @@
+/*
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 #ifndef SQUID_SSL_CERTIFICATE_DB_H
 #define SQUID_SSL_CERTIFICATE_DB_H
 
 #include "ssl/gadgets.h"
-#if HAVE_STRING
-#include <string>
-#endif
+
 #if HAVE_OPENSSL_OPENSSLV_H
 #include <openssl/opensslv.h>
 #endif
+#include <string>
 
 namespace Ssl
 {
@@ -178,3 +185,4 @@ private:
 
 } // namespace Ssl
 #endif // SQUID_SSL_CERTIFICATE_DB_H
+
