@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -54,7 +54,6 @@ urlInitialize(void)
     assert(0 == matchDomainName("foo.com", ".foo.com"));
     assert(0 == matchDomainName(".foo.com", ".foo.com"));
     assert(0 == matchDomainName("x.foo.com", ".foo.com"));
-    assert(0 == matchDomainName("y.x.foo.com", ".foo.com"));
     assert(0 != matchDomainName("x.foo.com", "foo.com"));
     assert(0 != matchDomainName("foo.com", "x.foo.com"));
     assert(0 != matchDomainName("bar.com", "foo.com"));
@@ -67,17 +66,6 @@ urlInitialize(void)
     assert(0 < matchDomainName("bfoo.com", "afoo.com"));
     assert(0 > matchDomainName("afoo.com", "bfoo.com"));
     assert(0 < matchDomainName("x-foo.com", ".foo.com"));
-
-    assert(0 == matchDomainName(".foo.com", ".foo.com", mdnRejectSubsubDomains));
-    assert(0 == matchDomainName("x.foo.com", ".foo.com", mdnRejectSubsubDomains));
-    assert(0 != matchDomainName("y.x.foo.com", ".foo.com", mdnRejectSubsubDomains));
-    assert(0 != matchDomainName(".x.foo.com", ".foo.com", mdnRejectSubsubDomains));
-
-    assert(0 == matchDomainName("*.foo.com", "x.foo.com", mdnHonorWildcards));
-    assert(0 == matchDomainName("*.foo.com", ".x.foo.com", mdnHonorWildcards));
-    assert(0 == matchDomainName("*.foo.com", ".foo.com", mdnHonorWildcards));
-    assert(0 != matchDomainName("*.foo.com", "foo.com", mdnHonorWildcards));
-
     /* more cases? */
 }
 
@@ -302,8 +290,6 @@ urlParse(const HttpRequestMethod& method, char *url, HttpRequest *request)
             *t = 0;
             strncpy((char *) host, t + 1, sizeof(host)-1);
             host[sizeof(host)-1] = '\0';
-            // Bug 4498: URL-unescape the login info after extraction
-            rfc1738_unescape(login);
         }
 
         /* Is there any host information? (we should eventually parse it above) */
@@ -701,21 +687,39 @@ urlMakeAbsolute(const HttpRequest * req, const char *relUrl)
     return (urlbuf);
 }
 
+/*
+ * matchDomainName() compares a hostname with a domainname according
+ * to the following rules:
+ *
+ *    HOST          DOMAIN        MATCH?
+ * ------------- -------------    ------
+ *    foo.com       foo.com         YES
+ *   .foo.com       foo.com         YES
+ *  x.foo.com       foo.com          NO
+ *    foo.com      .foo.com         YES
+ *   .foo.com      .foo.com         YES
+ *  x.foo.com      .foo.com         YES
+ *
+ *  We strip leading dots on hosts (but not domains!) so that
+ *  ".foo.com" is is always the same as "foo.com".
+ *
+ *  Return values:
+ *     0 means the host matches the domain
+ *     1 means the host is greater than the domain
+ *    -1 means the host is less than the domain
+ */
+
 int
-matchDomainName(const char *h, const char *d, uint flags)
+matchDomainName(const char *h, const char *d)
 {
     int dl;
     int hl;
 
-    const bool hostIncludesSubdomains = (*h == '.');
     while ('.' == *h)
         ++h;
 
     hl = strlen(h);
 
-    if (hl == 0)
-        return -1;
-
     dl = strlen(d);
 
     /*
@@ -753,20 +757,9 @@ matchDomainName(const char *h, const char *d, uint flags)
              * is a leading '.'.
              */
 
-            if ('.' == d[0]) {
-                if (flags & mdnRejectSubsubDomains) {
-                    // Check for sub-sub domain and reject
-                    while(--hl >= 0 && h[hl] != '.');
-                    if (hl < 0) {
-                        // No sub-sub domain found, but reject if there is a
-                        // leading dot in given host string (which is removed
-                        // before the check is started).
-                        return hostIncludesSubdomains ? 1 : 0;
-                    } else
-                        return 1; // sub-sub domain, reject
-                } else
-                    return 0;
-            } else
+            if ('.' == d[0])
+                return 0;
+            else
                 return 1;
         }
     }
@@ -774,13 +767,6 @@ matchDomainName(const char *h, const char *d, uint flags)
     /*
      * We found different characters in the same position (from the end).
      */
-
-    // If the h has a form of "*.foo.com" and d has a form of "x.foo.com"
-    // then the h[hl] points to '*', h[hl+1] to '.' and d[dl] to 'x'
-    // The following checks are safe, the "h[hl + 1]" in the worst case is '\0'.
-    if ((flags & mdnHonorWildcards) && h[hl] == '*' && h[hl + 1] == '.')
-        return 0;
-
     /*
      * If one of those character is '.' then its special.  In order
      * for splay tree sorting to work properly, "x-foo.com" must
